@@ -17,6 +17,7 @@ static struct {
  {"聲調拼音, 使用 asdf 選擇", "pinyin-asdf"},
 {"Dvorak", "dvorak"},
  {"IBM", "ibm"},
+ {"神通", "mitac"},
  {NULL, NULL}
 };
 
@@ -28,6 +29,7 @@ static GtkWidget *check_button_tsin_phrase_pre_select,
                  *check_button_phonetic_huge_tab,
                  *check_button_tsin_tone_char_input,
                  *check_button_tsin_tab_phrase_end,
+                 *check_button_tsin_tail_select_key,
                  *spinner_tsin_buffer_size;
 
 static GtkWidget *opt_kbm_opts;
@@ -40,6 +42,8 @@ static struct {
   {"CapsLock", TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock},
   {"Tab", TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Tab},
   {"Shift(限非 XIM)", TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift},
+  {"ShiftL(限非 XIM)", TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL},
+  {"ShiftR(限非 XIM)", TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR},
 };
 int tsin_eng_ch_swN = sizeof(tsin_eng_ch_sw) / sizeof(tsin_eng_ch_sw[0]);
 
@@ -101,6 +105,9 @@ static gboolean cb_ok( GtkWidget *widget,
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_tab_phrase_end)));
 
 
+  save_gcin_conf_int(TSIN_TAIL_SELECT_KEY,
+       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_tail_select_key)));
+
   tsin_buffer_size = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_tsin_buffer_size));
   save_gcin_conf_int(TSIN_BUFFER_SIZE, tsin_buffer_size);
 
@@ -440,15 +447,23 @@ void create_kbm_window()
      GTK_TOGGLE_BUTTON(check_button_tsin_tone_char_input), tsin_tone_char_input);
 
 
-  GtkWidget *hbox_tsin_tab_phrase_end = gtk_hbox_new(FALSE, 0);
+  GtkWidget *hbox_tsin_tab_phrase_end = gtk_hbox_new(FALSE, 1);
   gtk_box_pack_start (GTK_BOX (vbox_top), hbox_tsin_tab_phrase_end , TRUE, TRUE, 1);
-  GtkWidget *label_tsin_tab_phrase_end = gtk_label_new("詞音 Tab 斷詞");
+  GtkWidget *label_tsin_tab_phrase_end = gtk_label_new("詞音/Escape Tab 斷詞");
   gtk_box_pack_start (GTK_BOX (hbox_tsin_tab_phrase_end), label_tsin_tab_phrase_end , TRUE, TRUE, 0);
   check_button_tsin_tab_phrase_end = gtk_check_button_new ();
   gtk_box_pack_start (GTK_BOX (hbox_tsin_tab_phrase_end), check_button_tsin_tab_phrase_end, FALSE, FALSE, 0);
   gtk_toggle_button_set_active(
      GTK_TOGGLE_BUTTON(check_button_tsin_tab_phrase_end), tsin_tab_phrase_end);
 
+  GtkWidget *hbox_tsin_tail_select_key = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox_top), hbox_tsin_tail_select_key , TRUE, TRUE, 1);
+  GtkWidget *label_tsin_tail_select_key = gtk_label_new("同音字詞選擇按鍵在後");
+  gtk_box_pack_start (GTK_BOX (hbox_tsin_tail_select_key), label_tsin_tail_select_key , TRUE, TRUE, 0);
+  check_button_tsin_tail_select_key = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_tsin_tail_select_key), check_button_tsin_tail_select_key, FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(
+     GTK_TOGGLE_BUTTON(check_button_tsin_tail_select_key), tsin_tail_select_key);
 
   GtkWidget *frame_tsin_buffer_size = gtk_frame_new("詞音編輯緩衝區大小");
   gtk_box_pack_start (GTK_BOX (vbox_top), frame_tsin_buffer_size, FALSE, FALSE, 0);
