@@ -2,31 +2,30 @@
 #include "gtab.h"
 
 
-static GtkWidget *check_button_gtab_dup_select_bell,
-                 *check_button_gtab_auto_select_by_phrase,
-                 *check_button_gtab_press_full_auto_send,
-                 *check_button_gtab_pre_select,
-                 *check_button_gtab_disp_partial_match,
-                 *check_button_gtab_simple_win,
-                 *check_button_gtab_disp_key_codes,
-                 *check_button_gtab_disp_im_name,
-                 *check_button_gtab_invalid_key_in,
-                 *check_button_gtab_shift_phrase_key,
-                 *check_button_gtab_hide_row2,
-                 *check_button_gtab_in_row1,
-                 *check_button_gtab_capslock_in_eng,
-                 *check_button_root_style_use,
+static GtkWidget *check_button_root_style_use,
                  *check_button_gcin_pop_up_win,
-                 *check_button_gcin_inner_frame;
-#if TRAY_ENABLED
-                 *check_button_gcin_status_tray;
-#endif
+                 *check_button_gcin_inner_frame,
+                 *check_button_gcin_status_tray,
+                 *check_button_gcin_win_color_use;
 
 static GtkWidget *opt_spc_opts;
 
 static GtkWidget *gcin_kbm_window = NULL, *gcin_appearance_conf_window;
 static GtkClipboard *pclipboard;
 GtkWidget *main_window;
+static GdkColor gcin_win_gcolor_fg, gcin_win_gcolor_bg;
+
+typedef struct {
+  GdkColor *color;
+  char **color_str;
+  GtkColorSelectionDialog *color_selector;
+  char *title;
+} COLORSEL;
+
+COLORSEL colorsel[2] =
+  { {&gcin_win_gcolor_fg, &gcin_win_color_fg, "前景顏色"},
+    {&gcin_win_gcolor_bg, &gcin_win_color_bg, "背景顏色"}
+  };
 
 static gboolean close_application( GtkWidget *widget,
                                    GdkEvent  *event,
@@ -154,7 +153,7 @@ static void cb_ts_import()
    gtk_widget_show(file_selector);
 }
 
-static char utf8_edit[]=GCIN_SCRIPT_DIR"/utf8-edit";
+char utf8_edit[]=GCIN_SCRIPT_DIR"/utf8-edit";
 
 static void cb_ts_edit()
 {
@@ -209,27 +208,17 @@ static void cb_help()
   utf8_editor(DOC_DIR"/README");
 }
 
-struct {
-  char *str;
-  int num;
-} spc_opts[] = {
-  {"由 .gtab 指定", GTAB_space_auto_first_none},
-  {"按空白立即送出第一字(嘸蝦米、大易)", GTAB_space_auto_first_any},
-  {"按滿按空白送出第一字", GTAB_space_auto_first_full},
-  {"按滿按空白不送出第一字(倉頡, 行列)", GTAB_space_auto_first_nofull},
-  { NULL, 0},
-};
-
 static GtkWidget *spinner_gcin_font_size, *spinner_gcin_font_size_tsin_presel,
                  *spinner_gcin_font_size_symbol,*spinner_gcin_font_size_pho_near,
                  *spinner_gcin_font_size_tsin_pho_in, *spinner_gcin_font_size_gtab_in, *spinner_root_style_x,
                  *spinner_root_style_y, *font_sel;
 
+static GtkWidget *label_win_color_test, *event_box_win_color_test;
+
 static gboolean cb_appearance_conf_ok( GtkWidget *widget,
                                    GdkEvent  *event,
                                    gpointer   data )
 {
-
   int font_size = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_gcin_font_size));
   save_gcin_conf_int(GCIN_FONT_SIZE, font_size);
 
@@ -282,6 +271,17 @@ static gboolean cb_appearance_conf_ok( GtkWidget *widget,
   save_gcin_conf_int(GCIN_STATUS_TRAY, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_status_tray)));
 #endif
 
+  gchar *cstr = gtk_color_selection_palette_to_string(&gcin_win_gcolor_fg, 1);
+  dbg("color fg %s\n", cstr);
+  save_gcin_conf_str(GCIN_WIN_COLOR_FG, cstr);
+  g_free(cstr);
+
+  cstr = gtk_color_selection_palette_to_string(&gcin_win_gcolor_bg, 1);
+  dbg("color bg %s\n", cstr);
+  save_gcin_conf_str(GCIN_WIN_COLOR_BG, cstr);
+  g_free(cstr);
+
+  save_gcin_conf_int(GCIN_WIN_COLOR_USE, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_win_color_use)));
 
   send_gcin_message(GDK_DISPLAY(), CHANGE_FONT_SIZE);
   gtk_widget_destroy(gcin_appearance_conf_window); gcin_appearance_conf_window = NULL;
@@ -298,6 +298,54 @@ static gboolean close_appearance_conf_window( GtkWidget *widget,
 }
 
 
+static void cb_savecb_gcin_win_color_fg(GtkWidget *widget, gpointer user_data)
+{
+  COLORSEL *sel = user_data;
+  GtkColorSelectionDialog *color_selector = sel->color_selector;
+  GdkColor *col = sel->color;
+  gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(color_selector->colorsel), col);
+
+  if (sel->color == &gcin_win_gcolor_fg)
+    gtk_widget_modify_fg(label_win_color_test, GTK_STATE_NORMAL, col);
+  else {
+    gtk_widget_modify_bg(event_box_win_color_test, GTK_STATE_NORMAL, col);
+  }
+}
+
+static gboolean cb_gcin_win_color_fg( GtkWidget *widget,
+                                   gpointer   data)
+{
+  COLORSEL *sel = data;
+  GtkColorSelectionDialog *color_selector =
+  (GtkColorSelectionDialog *)gtk_color_selection_dialog_new (sel->title);
+
+  gdk_color_parse(*sel->color_str, sel->color);
+
+  gtk_color_selection_set_current_color(
+          GTK_COLOR_SELECTION(color_selector->colorsel),
+          sel->color);
+
+  g_signal_connect (GTK_OBJECT (color_selector->ok_button),
+                    "clicked",
+                    G_CALLBACK (cb_savecb_gcin_win_color_fg),
+                    (gpointer) sel);
+
+  sel->color_selector = color_selector;
+
+  g_signal_connect_swapped (GTK_OBJECT (color_selector->ok_button),
+                            "clicked",
+                            G_CALLBACK (gtk_widget_destroy),
+                            (gpointer) color_selector);
+
+  g_signal_connect_swapped (GTK_OBJECT (color_selector->cancel_button),
+                            "clicked",
+                            G_CALLBACK (gtk_widget_destroy),
+                            (gpointer) color_selector);
+
+  gtk_widget_show((GtkWidget*)color_selector);
+  return TRUE;
+}
+
 void create_appearance_conf_window()
 {
   if (gcin_appearance_conf_window) {
@@ -444,6 +492,40 @@ void create_appearance_conf_window()
   gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), check_button_gcin_status_tray, FALSE, FALSE, 0);
 #endif
 
+  GtkWidget *frame_win_color = gtk_frame_new("顏色選擇");
+  gtk_box_pack_start (GTK_BOX (vbox_top), frame_win_color, FALSE, FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (frame_win_color), 3);
+  GtkWidget *vbox_win_color = gtk_vbox_new (FALSE, 10);
+  gtk_container_add (GTK_CONTAINER (frame_win_color), vbox_win_color);
+
+  GtkWidget *hbox_win_color_use = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX(vbox_win_color), hbox_win_color_use, FALSE, FALSE, 0);
+  GtkWidget *label_win_color_use = gtk_label_new("使用");
+  gtk_box_pack_start (GTK_BOX(hbox_win_color_use), label_win_color_use, FALSE, FALSE, 0);
+  check_button_gcin_win_color_use = gtk_check_button_new ();
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_win_color_use),
+       gcin_win_color_use);
+  gtk_box_pack_start (GTK_BOX(hbox_win_color_use), check_button_gcin_win_color_use, FALSE, FALSE, 0);
+  event_box_win_color_test = gtk_event_box_new();
+  gtk_box_pack_start (GTK_BOX(vbox_win_color), event_box_win_color_test, FALSE, FALSE, 0);
+  label_win_color_test = gtk_label_new("測試目前狀態");
+  gtk_container_add (GTK_BOX(event_box_win_color_test), label_win_color_test);
+  GtkWidget *hbox_win_color_fbg = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX(vbox_win_color), hbox_win_color_fbg, FALSE, FALSE, 0);
+  GtkWidget *button_fg = gtk_button_new_with_label("前景顏色");
+  gtk_box_pack_start (GTK_BOX(hbox_win_color_fbg), button_fg, TRUE, TRUE, 0);
+  g_signal_connect (G_OBJECT (button_fg), "clicked",
+                    G_CALLBACK (cb_gcin_win_color_fg), &colorsel[0]);
+  GdkColor color;
+  gdk_color_parse(gcin_win_color_fg, &gcin_win_gcolor_fg);
+  gtk_widget_modify_fg(label_win_color_test, GTK_STATE_NORMAL, &gcin_win_gcolor_fg);
+  gdk_color_parse(gcin_win_color_bg, &gcin_win_gcolor_bg);
+  gtk_widget_modify_bg(event_box_win_color_test, GTK_STATE_NORMAL, &gcin_win_gcolor_bg);
+
+  GtkWidget *button_bg = gtk_button_new_with_label("背景顏色");
+  gtk_box_pack_start (GTK_BOX(hbox_win_color_fbg), button_bg, TRUE, TRUE, 0);
+  g_signal_connect (G_OBJECT (button_bg), "clicked",
+                    G_CALLBACK (cb_gcin_win_color_fg), &colorsel[1]);
 
   GtkWidget *hbox_cancel_ok = gtk_hbox_new (FALSE, 10);
   gtk_box_pack_start (GTK_BOX (vbox_top), hbox_cancel_ok, FALSE, FALSE, 0);
@@ -470,299 +552,6 @@ void create_appearance_conf_window()
   return;
 }
 
-static GtkWidget *gcin_gtab_conf_window;
-
-static gboolean cb_gtab_conf_ok( GtkWidget *widget,
-                                   GdkEvent  *event,
-                                   gpointer   data )
-{
-  save_gcin_conf_int(GTAB_DUP_SELECT_BELL,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_dup_select_bell)));
-
-  save_gcin_conf_int(GTAB_AUTO_SELECT_BY_PHRASE,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_auto_select_by_phrase)));
-
-  save_gcin_conf_int(GTAB_PRE_SELECT,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_pre_select)));
-
-  save_gcin_conf_int(GTAB_PRESS_FULL_AUTO_SEND,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_press_full_auto_send)));
-
-  save_gcin_conf_int(GTAB_DISP_PARTIAL_MATCH,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_disp_partial_match)));
-
-  save_gcin_conf_int(GTAB_SIMPLE_WIN,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_simple_win)));
-
-  save_gcin_conf_int(GTAB_DISP_KEY_CODES,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_disp_key_codes)));
-
-  save_gcin_conf_int(GTAB_DISP_IM_NAME,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_disp_im_name)));
-
-  save_gcin_conf_int(GTAB_INVALID_KEY_IN,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_invalid_key_in)));
-
-  save_gcin_conf_int(GTAB_SHIFT_PHRASE_KEY,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_shift_phrase_key)));
-
-  save_gcin_conf_int(GTAB_HIDE_ROW2,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_hide_row2)));
-
-  save_gcin_conf_int(GTAB_IN_ROW1,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_in_row1)));
-
-  save_gcin_conf_int(GTAB_CAPSLOCK_IN_ENG,
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_capslock_in_eng)));
-
-
-  int idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_spc_opts));
-  save_gcin_conf_int(GTAB_SPACE_AUTO_FIRST, spc_opts[idx].num);
-
-  send_gcin_message(GDK_DISPLAY(), CHANGE_FONT_SIZE);
-  gtk_widget_destroy(gcin_gtab_conf_window); gcin_gtab_conf_window = NULL;
-
-  return TRUE;
-}
-
-
-static gboolean close_gtab_conf_window( GtkWidget *widget,
-                                   GdkEvent  *event,
-                                   gpointer   data )
-{
-  gtk_widget_destroy(gcin_gtab_conf_window); gcin_gtab_conf_window = NULL;
-  return TRUE;
-}
-
-
-static GtkWidget *create_spc_opts()
-{
-  GtkWidget *hbox = gtk_hbox_new (FALSE, 1);
-  GtkWidget *label = gtk_label_new("空白鍵選項");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
-  opt_spc_opts = gtk_option_menu_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), opt_spc_opts, FALSE, FALSE, 0);
-  GtkWidget *menu_spc_opts = gtk_menu_new ();
-
-  int i, current_idx=0;
-
-  for(i=0; spc_opts[i].str; i++) {
-    GtkWidget *item = gtk_menu_item_new_with_label (spc_opts[i].str);
-
-    if (spc_opts[i].num == gtab_space_auto_first)
-      current_idx = i;
-
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu_spc_opts), item);
-  }
-
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_spc_opts), menu_spc_opts);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_spc_opts), current_idx);
-
-  return hbox;
-}
-
-
-void create_gtab_conf_window()
-{
-  if (gcin_gtab_conf_window) {
-    gtk_window_present(GTK_WINDOW(gcin_gtab_conf_window));
-    return;
-  }
-
-  load_setttings();
-
-    gcin_gtab_conf_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-
-    g_signal_connect (G_OBJECT (gcin_gtab_conf_window), "delete_event",
-                      G_CALLBACK (close_gtab_conf_window),
-                      NULL);
-
-    gtk_window_set_title (GTK_WINDOW (gcin_gtab_conf_window), "倉頡/行列/嘸蝦米/大易設定");
-    gtk_container_set_border_width (GTK_CONTAINER (gcin_gtab_conf_window), 3);
-
-    GtkWidget *vbox_top = gtk_vbox_new (FALSE, 10);
-    gtk_container_add (GTK_CONTAINER (gcin_gtab_conf_window), vbox_top);
-
-
-    GtkWidget *frame_gtab = gtk_frame_new("倉頡/行列/嘸蝦米/大易");
-    gtk_container_set_border_width (GTK_CONTAINER (frame_gtab), 5);
-    gtk_box_pack_start (GTK_BOX (vbox_top), frame_gtab, FALSE, FALSE, 0);
-    GtkWidget *vbox_gtab = gtk_vbox_new (FALSE, 10);
-    gtk_container_add (GTK_CONTAINER (frame_gtab), vbox_gtab);
-    gtk_container_set_border_width (GTK_CONTAINER (vbox_gtab), 10);
-
-
-    GtkWidget *hbox_gtab_dup_select_bell = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_dup_select_bell, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_sele = gtk_label_new("重複字選擇鈴聲");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_dup_select_bell), label_gtab_sele,  FALSE, FALSE, 0);
-    check_button_gtab_dup_select_bell = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_dup_select_bell),check_button_gtab_dup_select_bell,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_dup_select_bell),
-       gtab_dup_select_bell);
-
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), create_spc_opts(), FALSE, FALSE, 0);
-
-    GtkWidget *hbox_gtab_auto_select_by_phrase = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_auto_select_by_phrase, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_auto_select = gtk_label_new("由詞庫自動選擇字");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_auto_select_by_phrase), label_gtab_auto_select,  FALSE, FALSE, 0);
-    check_button_gtab_auto_select_by_phrase = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_auto_select_by_phrase),check_button_gtab_auto_select_by_phrase,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_auto_select_by_phrase),
-       gtab_auto_select_by_phrase);
-
-
-    GtkWidget *hbox_gtab_pre_select = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_pre_select, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_pre_select = gtk_label_new("預覽/預選 字");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_pre_select), label_gtab_pre_select,  FALSE, FALSE, 0);
-    check_button_gtab_pre_select = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_pre_select),check_button_gtab_pre_select,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_pre_select),
-       gtab_pre_select);
-
-
-    GtkWidget *hbox_gtab_press_full_auto_send = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_press_full_auto_send, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_gtab_press_full_auto_send = gtk_label_new("按滿自動送字");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_press_full_auto_send), label_gtab_gtab_press_full_auto_send,  FALSE, FALSE, 0);
-    check_button_gtab_press_full_auto_send = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_press_full_auto_send), check_button_gtab_press_full_auto_send,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_press_full_auto_send),
-       gtab_press_full_auto_send);
-
-
-    GtkWidget *hbox_gtab_disp_partial_match = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_disp_partial_match, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_gtab_disp_partial_match = gtk_label_new("預選列中顯示部份符合的字");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_disp_partial_match), label_gtab_gtab_disp_partial_match,  FALSE, FALSE, 0);
-    check_button_gtab_disp_partial_match = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_disp_partial_match), check_button_gtab_disp_partial_match,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_disp_partial_match),
-       gtab_disp_partial_match);
-
-
-    GtkWidget *hbox_gtab_simple_win = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_simple_win, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_gtab_simple_win = gtk_label_new("較小視窗");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_simple_win), label_gtab_gtab_simple_win,  FALSE, FALSE, 0);
-    check_button_gtab_simple_win = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_simple_win), check_button_gtab_simple_win,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_simple_win),
-       gtab_simple_win);
-
-
-    GtkWidget *hbox_gtab_disp_key_codes = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_disp_key_codes, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_gtab_disp_key_codes = gtk_label_new("顯示拆字");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_disp_key_codes), label_gtab_gtab_disp_key_codes,  FALSE, FALSE, 0);
-    check_button_gtab_disp_key_codes = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_disp_key_codes), check_button_gtab_disp_key_codes,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_disp_key_codes),
-       gtab_disp_key_codes);
-
-
-    GtkWidget *hbox_gtab_disp_im_name = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_disp_im_name, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_gtab_disp_im_name = gtk_label_new("顯示輸入法名稱");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_disp_im_name), label_gtab_gtab_disp_im_name,  FALSE, FALSE, 0);
-    check_button_gtab_disp_im_name = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_disp_im_name), check_button_gtab_disp_im_name,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_disp_im_name),
-       gtab_disp_im_name);
-
-
-    GtkWidget *hbox_gtab_invalid_key_in = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_invalid_key_in, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_gtab_invalid_key_in = gtk_label_new("錯誤鍵進入(傳統)");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_invalid_key_in), label_gtab_gtab_invalid_key_in,  FALSE, FALSE, 0);
-    check_button_gtab_invalid_key_in = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_invalid_key_in), check_button_gtab_invalid_key_in,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_invalid_key_in),
-       gtab_invalid_key_in);
-
-
-    GtkWidget *hbox_gtab_shift_phrase_key = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_shift_phrase_key, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_gtab_shift_phrase_key = gtk_label_new("Shift 用來輸入片語(Alt-Shift)");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_shift_phrase_key), label_gtab_gtab_shift_phrase_key,  FALSE, FALSE, 0);
-    check_button_gtab_shift_phrase_key = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_shift_phrase_key), check_button_gtab_shift_phrase_key,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_shift_phrase_key),
-       gtab_shift_phrase_key);
-
-
-    GtkWidget *hbox_gtab_hide_row2 = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_hide_row2, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_hide_row2 = gtk_label_new("隱藏第二列(輸入鍵…)");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_hide_row2), label_gtab_hide_row2,  FALSE, FALSE, 0);
-    check_button_gtab_hide_row2 = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_hide_row2), check_button_gtab_hide_row2,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_hide_row2),
-       gtab_hide_row2);
-
-
-    GtkWidget *hbox_gtab_in_row1 = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_in_row1, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_in_row1 = gtk_label_new("輸入鍵顯示移至第一列");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_in_row1), label_gtab_in_row1,  FALSE, FALSE, 0);
-    check_button_gtab_in_row1 = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_in_row1), check_button_gtab_in_row1,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_in_row1),
-       gtab_in_row1);
-
-
-    GtkWidget *hbox_gtab_capslock_in_eng = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_gtab), hbox_gtab_capslock_in_eng, FALSE, FALSE, 0);
-    GtkWidget *label_gtab_capslock_in_eng = gtk_label_new("CapsLock 打開輸入英數");
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_capslock_in_eng), label_gtab_capslock_in_eng,  FALSE, FALSE, 0);
-    check_button_gtab_capslock_in_eng = gtk_check_button_new ();
-    gtk_box_pack_start (GTK_BOX (hbox_gtab_capslock_in_eng), check_button_gtab_capslock_in_eng,  FALSE, FALSE, 0);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_capslock_in_eng),
-       gtab_capslock_in_eng);
-
-
-    GtkWidget *hbox_cancel_ok = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_cancel_ok, FALSE, FALSE, 0);
-
-    GtkWidget *button_cancel = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-    gtk_box_pack_start (GTK_BOX (hbox_cancel_ok), button_cancel, TRUE, TRUE, 0);
-
-    g_signal_connect (G_OBJECT (button_cancel), "clicked",
-                              G_CALLBACK (close_gtab_conf_window),
-                              G_OBJECT (gcin_gtab_conf_window));
-
-    GtkWidget *button_ok = gtk_button_new_from_stock (GTK_STOCK_OK);
-    gtk_box_pack_start (GTK_BOX (hbox_cancel_ok), button_ok, TRUE, TRUE, 0);
-
-    g_signal_connect_swapped (G_OBJECT (button_ok), "clicked",
-                              G_CALLBACK (cb_gtab_conf_ok),
-                              G_OBJECT (gcin_gtab_conf_window));
-
-    GTK_WIDGET_SET_FLAGS (button_ok, GTK_CAN_DEFAULT);
-    gtk_widget_grab_default (button_ok);
-
-    gtk_widget_show_all (gcin_gtab_conf_window);
-
-    return;
-}
-
-
 
 static void cb_appearance_conf()
 {
