@@ -54,8 +54,6 @@ void loadIC();
 IC *FindIC(CARD16 icid);
 extern ClientState *current_CS;
 
-
-
 enum {
   InputStyleOverSpot = 1,
   InputStyleRoot = 2,
@@ -69,15 +67,14 @@ enum {
   Windows_Space=3,
 } IM_TOGGLE_KEYS;
 
-
-
 enum {
   TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock=1,
   TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Tab=2,
   TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift=4,
+  TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL=8,
+  TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR=16,
 } TSIN_CHINESE_ENGLISH_TOGGLE_KEY;
 
-
 enum {
   TSIN_SPACE_OPT_SELECT_CHAR = 1,
   TSIN_SPACE_OPT_FLUSH_BUFFER = 2,
@@ -117,6 +114,8 @@ gboolean utf8_eq(char *a, char *b);
 gboolean utf8_str_eq(char *a, char *b, int len);
 void utf8cpyN(char *t, char *s, int N);
 int utf8_str_N(char *str);
+void utf8cpyn(char *t, char *s, int n);
+void utf8cpy_bytes(char *t, char *s, int n);
 
 void get_gcin_dir(char *tt);
 Atom get_gcin_atom(Display *dpy);
@@ -129,6 +128,8 @@ void set_label_font_size(GtkWidget *label, int size);
 void send_gcin_message(Display *dpy, char *s);
 void check_CS();
 void get_win_size(GtkWidget *win, int *width, int *height);
+void change_win_fg_bg(GtkWidget *win, GtkWidget *label);
+void set_no_focus();
 
 
 #define BITON(flag, bit) ((flag) & (bit))
