@@ -1,12 +1,12 @@
-#define MAX_CIN_PHR (32*CH_SZ + 1)
+#define MAX_CIN_PHR (100*CH_SZ + 1)
 
 
 
 typedef enum {
   GTAB_space_auto_first_none=0,   // use the value set by .cin
   GTAB_space_auto_first_any=1,    // boshiamy, dayi
-  GTAB_space_auto_first_full=2,   // cj
-  GTAB_space_auto_first_nofull=4  // windows ar30
+  GTAB_space_auto_first_full=2,   // simplex
+  GTAB_space_auto_first_nofull=4  // windows ar30 cj
 } GTAB_space_pressed_E;
 
 typedef struct {
@@ -54,7 +54,7 @@ struct TableHead {
 #define KeyBits (6)
 #define MAX_GTAB_KEYS (1<<KeyBits)
 
-#define MAX_GTAB_NUM_KEY (14)
+#define MAX_GTAB_NUM_KEY (16)
 #define MAX_SELKEY 16
 
 #define MAX_TAB_KEY_NUM 5
@@ -98,7 +98,14 @@ extern INMD inmd[MAX_GTAB_NUM_KEY+1];
 
 u_int64_t CONVT2(INMD *inmd, int i);
 extern INMD *cur_inmd;
+void load_gtab_list();
 
 #define LAST_K_bitN (cur_inmd->key64 ? 54:24)
 
 #define GTAB_LIST "gtab.list"
+
+#if 1
+#define NEED_SWAP (__BYTE_ORDER == __BIG_ENDIAN && 0)
+#else
+#define NEED_SWAP (1)
+#endif
