@@ -3,10 +3,12 @@
 #include "pho.h"
 #include "tsin.h"
 #include "gcin-conf.h"
+#include <math.h>
 
 typedef struct {
   int start;
-  float score;
+  int usecount;
+  short match_phr_N, no_match_ch_N;
   TSIN_PARSE best[MAX_PH_BF_EXT+1];
 } CACHE ;
 
@@ -23,44 +25,49 @@ static CACHE *cache_lookup(int start)
   return NULL;
 }
 
-static void add_cache(int start, float score, TSIN_PARSE *out)
+static void add_cache(int start, int usecount, TSIN_PARSE *out,
+                      short match_phr_N, short no_match_ch_N)
 {
   cache[cacheN].start = start;
-  cache[cacheN].score = score;
+  cache[cacheN].usecount = usecount;
+  cache[cacheN].match_phr_N = match_phr_N;
+  cache[cacheN].no_match_ch_N = no_match_ch_N;
   memcpy(cache[cacheN].best, out, sizeof(TSIN_PARSE) * (c_len - start));
   cacheN++;
 }
 
-float tsin_parse_recur(int start, TSIN_PARSE *out)
+#define DBG (0)
+
+int tsin_parse_recur(int start, TSIN_PARSE *out,
+                     short *r_match_phr_N, short *r_no_match_ch_N)
 {
   int plen;
-  float bestscore = 0;
+  double bestscore = -1;
+  int bestusecount = 0;
+  *r_match_phr_N = 0;
+  *r_no_match_ch_N = c_len - start;
 
-#if 0
-  /* if there is no phrase, out is not updated in the recursive parse,
-     so we need this */
-  int i;
-  for(i=0; i < c_len - start; i++) {
-    out[i].len = 1;
-    out[i].start = start + i;
-    utf8cpy(out[i].str, chpho[start + i].ch);
-  }
-#endif
 
   for(plen=1; start + plen <= c_len && plen <= MAX_PHRASE_LEN; plen++) {
+    if (plen > 1 && (chpho[start+plen-1].flag & FLAG_CHPHO_PHRASE_USER_HEAD))
+      break;
     phokey_t pp[MAX_PHRASE_LEN + 1];
     int sti, edi;
     TSIN_PARSE pbest[MAX_PH_BF_EXT+1];
-    float pbestscore = 0;
+    int maxusecount = 0;
     int remlen;
+    short match_phr_N=0, no_match_ch_N = 1;
 
     bzero(pbest, sizeof(TSIN_PARSE) * c_len);
 
-    pbest[0].len = 1;
+    pbest[0].len = plen;
     pbest[0].start = start;
-    utf8cpy(pbest[0].str, chpho[start].ch);
-#if 0
-    dbg("hh "); utf8_putchar(chpho[start].ch); dbg("\n");
+    int i, ofs;
+    for(ofs=i=0; i < plen; i++)
+      ofs += utf8cpy(pbest[0].str + ofs, chpho[start + i].ch);
+
+#if DBG
+    dbg("st:%d hh plen:%d ", start, plen);utf8_putchar(chpho[start].ch); dbg("\n");
 #endif
 
     extract_pho(start, plen, pp);
@@ -71,6 +78,7 @@ float tsin_parse_recur(int start, TSIN_PARSE *out)
       goto next;
     }
 
+
     for (;sti < edi; sti++) {
       phokey_t mtk[MAX_PHRASE_LEN];
       char mtch[MAX_PHRASE_LEN*CH_SZ+1];
@@ -96,78 +104,89 @@ float tsin_parse_recur(int start, TSIN_PARSE *out)
       if (i < plen)
         continue;
 
-      int score;
-
-      score = plen;
       if (match_len > plen) {
-#if 0
-        if (start + plen == c_len)
-          pbest[0].flag |= FLAG_TSIN_PARSE_PARTIAL;
-
-        if (pbestscore < score) {
-#if 1
-          dbg("start:%d flag:%x %d part plen %d  sc:%d,%d ",
-            start, pbest[0].flag, sti, plen, pbestscore, score);
-          utf8_putchar(chpho[start].ch); dbg(" %d\n", pbest[0].len);
-#endif
-//          pbestscore = score;
-        }
-#endif
         continue;
       }
 
-      score = usecount + 50 * plen * plen;
-
-      if (pbestscore >= score)
+      if (usecount < maxusecount)
         continue;
 
       pbest[0].len = plen;
-      pbestscore = score;
+      maxusecount = usecount;
       utf8cpyN(pbest[0].str, mtch, plen);
       pbest[0].flag |= FLAG_TSIN_PARSE_PHRASE;
-#if 0
-      utf8_putcharn(mtch, plen);
-      dbg("   plen %d score:%d,%d  usecount:%d  start:%d sti:%d\n",
-        plen, pbestscore, score, usecount, start, sti);
-#endif
-    }
 
-#if 0
-    // no match phrase was found
-    if (pbestscore == 0.0) {
-      if (plen > 1)
-        break;
-    }
+      match_phr_N = 1;
+      no_match_ch_N = 0;
+#if DBG
+      utf8_putcharn(mtch, plen);
+      dbg("   plen %d usecount:%d  ", plen, usecount);
+        utf8_putcharn(mtch, plen);
+      dbg("\n");
 #endif
+    }
 
 next:
     remlen = c_len - (start + plen);
 
+
     if (remlen) {
       int next = start + plen;
-      int uc;
       CACHE *pca;
 
+      short smatch_phr_N, sno_match_ch_N;
+      int uc;
+
       if (pca = cache_lookup(next)) {
-        uc = pca->score;
+        uc = pca->usecount;
+        smatch_phr_N = pca->match_phr_N;
+        sno_match_ch_N = pca->no_match_ch_N;
         memcpy(&pbest[1], pca->best, (c_len - next) * sizeof(TSIN_PARSE));
       } else {
-        uc = tsin_parse_recur(next, &pbest[1]);
-        add_cache(next, uc, &pbest[1]);
+        uc = tsin_parse_recur(next, &pbest[1], &smatch_phr_N, &sno_match_ch_N);
+//        dbg("   gg %d\n", smatch_phr_N);
+        add_cache(next, uc, &pbest[1], smatch_phr_N, sno_match_ch_N);
       }
 
-      pbestscore += uc;
+      match_phr_N += smatch_phr_N;
+      no_match_ch_N += sno_match_ch_N;
+      maxusecount += uc;
     }
 
-    if (bestscore < pbestscore) {
-      bestscore = pbestscore;
+    double score = log(maxusecount + 10) /
+      pow(match_phr_N + 0.05, 6) / pow(no_match_ch_N + 0.01, 8);
+
+    utf8_putchar(chpho[start].ch);
+#if DBG
+    dbg("%d zz muse:%d ma:%d noma:%d  score:%.4e %.4e\n", plen,
+        maxusecount, match_phr_N, no_match_ch_N, score, bestscore);
+#endif
+    if (score > bestscore) {
+#if DBG
+      dbg("is best org %.4e\n", bestscore);
+#endif
+      bestscore = score;
       memcpy(out, pbest, sizeof(TSIN_PARSE) * (c_len - start));
+
+#if DBG
+      dbg("ggg %d  ", start);
+      int i;
+      for(i=0;  i < c_len - start; i++) {
+        utf8_putcharn(out[i].str, out[i].len);
+      }
+      dbg("\n");
+#endif
+
+      bestusecount = maxusecount;
+      *r_match_phr_N = match_phr_N;
+      *r_no_match_ch_N = no_match_ch_N;
     }
   }
 
-  return bestscore;
+  return bestusecount;
 }
 
+void disp_ph_sta_idx(int idx);
 
 void tsin_parse(TSIN_PARSE out[])
 {
@@ -177,7 +196,8 @@ void tsin_parse(TSIN_PARSE out[])
   cache = tmalloc(CACHE, c_len);
   cacheN = 0;
 
-  tsin_parse_recur(0, out);
+  short smatch_phr_N, sno_match_ch_N;
+  tsin_parse_recur(0, out, &smatch_phr_N, &sno_match_ch_N);
 
   int i, ofsi;
 
@@ -199,35 +219,14 @@ void tsin_parse(TSIN_PARSE out[])
 
       ofsi++;
     }
-#if 0
-    dbg("%d] %s  %d  flag:%x\n", i,  out[i].str, out[i].len, out[i].flag);
-#endif
   }
-#if 0
-  for(i=0; i < c_len; i++)
-    dbg("psta %d %x\n", chpho[i].psta, chpho[i].flag);
-#endif
 
-  i--;
-  if ((out[i].flag & FLAG_TSIN_PARSE_PHRASE) && c_len - out[i].start > 1) {
-    ph_sta = out[i].start;
+  int ph_sta_idx = ph_sta;
+  if (chpho[c_len-1].psta>=0 && c_len - chpho[c_len-1].psta > 1) {
+    ph_sta_idx = chpho[c_len-1].psta;
   }
-#if 0
-  else {
-    for(; i>=0 ; i--)
-      if (out[i].flag & FLAG_TSIN_PARSE_PARTIAL)
-        break;
 
-    if (i < 0)
-      ph_sta = -1;
-    else
-      ph_sta = out[i].start;
-  }
-#endif
-
-#if 0
-  dbg("%d ph_sta: %d\n",i, ph_sta);
-#endif
+  disp_ph_sta_idx(ph_sta_idx);
 
   free(cache); cache = NULL;
 }
