@@ -133,6 +133,7 @@ static void chpho_extract(CHPHO *chph, int len, phokey_t *pho, char *ch, char *o
 void inc_pho_count(phokey_t key, int ch_idx);
 int ch_key_to_ch_pho_idx(phokey_t phkey, char *big5);
 void inc_dec_tsin_use_count(phokey_t *pho, char *ch, int prlen, int N);
+void lookup_gtabn(char *ch);
 
 static void putbuf(int len)
 {
@@ -202,6 +203,7 @@ static void putbuf(int len)
 
   tt[idx]=0;
   send_text(tt);
+  lookup_gtabn(tt);
 }
 
 
@@ -211,20 +213,10 @@ static void prbuf()
 {
   int i;
 
-//  dbg("prbuf\n");
-
   for(i=0; i < c_len; i++) {
     disp_char_chbuf(i);
   }
 
-#if 0
-  if (c_len < MAX_PH_BF_EXT)
-    hide_char(c_len);
-
-  if (c_len+1 < MAX_PH_BF_EXT)
-    hide_char(c_len+1);
-#endif
-
   drawcursor();
 }
 
@@ -293,18 +285,22 @@ void clr_in_area_pho_tsin();
 void close_win_pho_near();
 void compact_win0_x();
 
-void tsin_reset_in_pho()
+void tsin_reset_in_pho0()
 {
-  clrin_pho_tsin();
-  prbuf();
+//  prbuf();
   clr_in_area_pho_tsin();
   close_selection_win();
   pre_selN = 0;
   drawcursor();
-
   close_win_pho_near();
 }
 
+void tsin_reset_in_pho()
+{
+  clrin_pho_tsin();
+  tsin_reset_in_pho0();
+}
+
 gboolean flush_tsin_buffer()
 {
   tsin_reset_in_pho();
@@ -460,8 +456,10 @@ static void save_phrase()
 static void set_fixed(int idx, int len)
 {
   int i;
-  for(i=idx; i < idx+len; i++)
+  for(i=idx; i < idx+len; i++) {
     chpho[i].flag |= FLAG_CHPHO_FIXED;
+    chpho[i].flag &= ~FLAG_CHPHO_PHRASE_USER_HEAD;
+  }
 }
 
 #define PH_SHIFT_N (tsin_buffer_size - 1)
@@ -481,7 +479,7 @@ static void shift_ins()
      int fixedlen = c_len - 10;
      if (fixedlen <= 0)
        fixedlen = 1;
-     set_fixed(0, Min(c_len, fixedlen));
+     set_fixed(0, fixedlen);
 
      ofs = 1;
      putbuf(ofs);
@@ -830,6 +828,8 @@ static u_char scanphr(int chpho_idx, int plen, gboolean pho_incr)
   return maxlen;
 }
 
+void hide_selections_win();
+
 static void disp_pre_sel_page()
 {
 
@@ -839,8 +839,10 @@ static void disp_pre_sel_page()
     return;
   }
 
-  if (pre_selN==0 || (pre_selN==1 && pre_sel[0].len<=2))
+  if (pre_selN==0 || (pre_selN==1 && pre_sel[0].len<=2) || ph_sta < 0) {
+    hide_selections_win();
     return;
+  }
 
   clear_sele();
 
@@ -855,7 +857,6 @@ static void disp_pre_sel_page()
   disp_selections(ph_sta);
 }
 
-void hide_selections_win();
 static void close_selection_win()
 {
   hide_selections_win();
@@ -911,8 +912,9 @@ static void clear_disp_ph_sta()
 
 void draw_underline(int index);
 
-static void disp_ph_sta()
+void disp_ph_sta_idx(int idx)
 {
+//  dbg("ph_sta:%d\n", ph_sta);
   clear_disp_ph_sta();
 
   if (ph_sta < 0)
@@ -920,12 +922,15 @@ static void disp_ph_sta()
 
   int i;
 
-  for(i=ph_sta; i < c_idx; i++) {
+  for(i=idx; i < c_idx; i++) {
     draw_underline(i);
   }
 }
 
-
+void disp_ph_sta()
+{
+  disp_ph_sta_idx(ph_sta);
+}
 
 void ch_pho_cpy(CHPHO *pchpho, char *utf8, phokey_t *phos, int len)
 {
@@ -1209,13 +1214,21 @@ int feedkey_pp(KeySym xkey, int kbstate)
      return 0;
    }
 
-
+   int o_sel_pho = sel_pho;
    close_win_pho_near();
 
    switch (xkey) {
      case XK_Escape:
-        tsin_reset_in_pho();
-        return 1;
+       tsin_reset_in_pho0();
+       if (!typ_pho[0] && !typ_pho[1] && !typ_pho[2] && !typ_pho[3]) {
+         if (!c_len)
+           return 0;
+
+         if (!o_sel_pho && tsin_tab_phrase_end)
+           goto tab_phrase_end;
+       }
+       tsin_reset_in_pho();
+       return 1;
      case XK_Return:
      case XK_KP_Enter:
         if (shift_m) {
@@ -1275,14 +1288,21 @@ int feedkey_pp(KeySym xkey, int kbstate)
         } else
           return 0;
      case XK_Tab:
+        close_selection_win();
         if (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Tab) {
-          close_selection_win();
           tsin_toggle_eng_ch();
           return 1;
         }
 
         if (tsin_tab_phrase_end) {
-          ph_sta = -1;
+tab_phrase_end:
+          if (c_idx==c_len)
+            chpho[c_idx-1].flag |= FLAG_CHPHO_PHRASE_USER_HEAD;
+          else
+            chpho[c_idx].flag |= FLAG_CHPHO_PHRASE_USER_HEAD;
+#if 1
+           call_tsin_parse();
+#endif
           return 1;
         } else {
           if (c_len) {
@@ -1556,7 +1576,7 @@ other_keys:
 #endif
            }
 
-           chpho[i].flag |= FLAG_CHPHO_FIXED;
+           set_fixed(i, 1);
 #if 1
            call_tsin_parse();
 #endif
@@ -1701,9 +1721,10 @@ llll2:
 
      disp_in_area_pho_tsin();
 
-     if (pre_selN && scanphr(ph_sta, c_idx - ph_sta, TRUE)) {
-       disp_pre_sel_page();
-     }
+     if (pre_selN)
+       scanphr(ph_sta, c_idx - ph_sta, TRUE);
+
+     disp_pre_sel_page();
 
      key = pho2key(typ_pho);
 
@@ -1891,8 +1912,13 @@ int feedkey_pp_release(KeySym xkey, int kbstate)
   switch (xkey) {
      case XK_Shift_L:
      case XK_Shift_R:
-        if (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift &&
-            current_time() - key_press_time < 300000) {
+        if (
+(  (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift) ||
+   (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL
+     && xkey == XK_Shift_L) ||
+   (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR
+     && xkey == XK_Shift_R))
+          &&  current_time() - key_press_time < 300000) {
           key_press_time = 0;
           close_selection_win();
           tsin_toggle_eng_ch();
